/*----------------------------------------------------------------------------------------------------

--

-- File name	 : fifo_sdram_interface_controller.V

--					 :

-- Title			 : FIFO -- SDRAM CONTROLLER

--					 :

-- Purpose		 : Controls data transfer between the SDRAM controller and the FIFO

--					 :

-- IncludeFiles :

--					 :

-- Created On	 : August 23, 2001

--					 :

-- Comments		 :

--					 : This is designed based on each burst containing 8 data itesms

--					 :

--					 :

--					 :

--					 :

--					 :  For the Spartan 2 Reference Board

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

-- Conventions  : - Port names are upper case.

--					 : - Internal signals and variables are lower case.

--					 : - Active low signals are identified with '_n' or '_N'.

--					 : - constants begin with lower case 'c', rest upper case

--					 : - states begin with lower case 's', rest upper case

--					 : - aliases begin with lower case 'a', rest upper case

--					 :

--					 : Tab setting = 3

--					 : Right Margin = 100

--					 :

-- Limitations  : For use in Xilinx FPGA

--					 :

-- Developers	 : Duncan Allen

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--	  Version No 	| Author	 	| Changes Made:							| Mod. Date:

--		v1.0			| da			| First write								| 23/8/01

----------------------------------------------------------------------------------------------------*/

`timescale 1 ns/1 ps



module fifo_sdram_interface_controller (

		CLK, 

		RESET, 

		BEGIN_WRITE,

      BEGIN_WRITE_OK,

		BEGIN_READ, 

		READ_UNDERWAY,

		WRITE_UNDERWAY,

		REN,

		WEN,

		TRANSFER,

		DVLD,

		READ_COMPLETE, 

		WRITE_COMPLETE

		);



input CLK;

input RESET;

input BEGIN_WRITE;

input BEGIN_WRITE_OK;

input BEGIN_READ;

output READ_UNDERWAY;

output WRITE_UNDERWAY;

output REN;

output WEN;

input TRANSFER;

input DVLD;

output READ_COMPLETE;

output WRITE_COMPLETE;



reg READ_UNDERWAY;

reg WRITE_UNDERWAY;

reg REN;

reg WEN;

reg READ_COMPLETE;

reg WRITE_COMPLETE;





// Internal signals

integer state;



// Define the states

parameter sIDLE = 1;

parameter sREAD_WAIT = 2;

parameter sREAD = 3;

parameter sREAD_FINISH = 4;

parameter sWRITE_WAIT = 5;

parameter sWRITE = 6;

parameter sWRITE_FINISH = 7;





// The state machine

always @ (posedge CLK or posedge RESET)

	begin

	if (RESET)

		begin

		READ_UNDERWAY <= 1'b0;

		WRITE_UNDERWAY <= 1'b0;

		REN <= 1'b0;

		WEN <= 1'b0;

		READ_COMPLETE <= 1'b0;

		WRITE_COMPLETE <= 1'b0;

		state <= sIDLE;

		end

	else

		case (state)

			sIDLE : begin

				if (BEGIN_WRITE_OK)

					begin

					WRITE_UNDERWAY <= 1'b1;

					state <= sWRITE_WAIT;

					end

				else if (BEGIN_READ && !BEGIN_WRITE)   // A write needing padding may be underway

					begin

					READ_UNDERWAY <= 1'b1;

					state <= sREAD_WAIT;

					end

				else

					begin

					state <= sIDLE;

					WRITE_UNDERWAY <= 1'b0;

					READ_UNDERWAY <= 1'b0;

					REN <= 1'b0;

					WEN <= 1'b0;

					READ_COMPLETE <= 1'b0;

					WRITE_COMPLETE <= 1'b0;

					end

			end

			

			sWRITE_WAIT : begin			 

				WEN <= 1'b1;

				if 	(TRANSFER)

					state <= sWRITE;

				else

					state <= sWRITE_WAIT;

			 end

			 

			 sWRITE : begin

				 if (TRANSFER)

					 begin

					 WEN <= 1'b1;

					 WRITE_UNDERWAY <= 1'b1;

					 WRITE_COMPLETE <= 1'b0;

					 state <= sWRITE;

					 end

				else

					begin

					WEN <= 1'b0;

					WRITE_UNDERWAY <= 1'b0;

					WRITE_COMPLETE <= 1'b1;

					state <= sWRITE_FINISH;

					end

			end



			sWRITE_FINISH : begin

				if(~BEGIN_WRITE)

					begin

					WRITE_COMPLETE <= 1'b0;

					state <= sIDLE;

					end

				else

					state <=sWRITE_FINISH;

			end

			sREAD_WAIT : begin

				REN <= 1'b1;

				if (DVLD)

					state <= sREAD;

				else

					state <= sREAD_WAIT;

			end



			sREAD : begin

				if (DVLD)

					begin

					REN <= 1'b0;

					READ_UNDERWAY <= 1'b1;

					READ_COMPLETE <= 1'b0;

					state <= sREAD;

					end

				else

					begin

					REN <= 1'b0;

					READ_UNDERWAY <= 1'b0;

					READ_COMPLETE <= 1'b1;

					state <= sREAD_FINISH;

					end

			end



			sREAD_FINISH : begin

				if (~BEGIN_READ)

					begin

					READ_COMPLETE <= 1'b0;

					state <= sIDLE;

					end

				else

					state <= sREAD_FINISH;

			end



			default : begin

				state <= sIDLE;

			end

		endcase

	end





endmodule

