/*----------------------------------------------------------------------------------------------------

--

-- File name	 : read_fifo.V

--					 :

-- Title			 : READ FIFO

--					 :

-- Purpose		 : READS data and byte enables from SDRAM to PCI

--					 :

-- IncludeFiles :

--					 :

-- Created On	 : August 23, 2001

--					 :

-- Comments		 :

--					 : This is designed based on each burst containing 8 data itesms

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :   For the Spartan 2 Reference Board

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

-- Conventions  : - Port names are upper case.

--					 : - Internal signals and variables are lower case.

--					 : - Active low signals are identified with '_n' or '_N'.

--					 : - constants begin with lower case 'c', rest upper case

--					 : - states begin with lower case 's', rest upper case

--					 : - aliases begin with lower case 'a', rest upper case

--					 :

--					 : Tab setting = 3

--					 : Right Margin = 100

--					 :

-- Limitations  : For use in Xilinx FPGA

--					 :

-- Developers	 : Duncan Allen

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--	  Version No 	| Author	 	| Changes Made:							| Mod. Date:

--		v1.0			| da			| First write								| 23/8/01

----------------------------------------------------------------------------------------------------*/







// This is a simple synchronous FIFO.

// It assumes that each burst of data will be 8 units long and that

// each burst will be emptied before a new one is loaded



// The read FIFO is the same as the write FIFO except there are no Byte Enables



`timescale 1 ns/1 ps



module read_fifo

	(	CLK,

		RESET,

		PUSH,

		POP,

		FLUSH,

		WRITE_DATA_COUNT,

		READ_DATA_COUNT,

		DATA_OUT,

		DATA_IN

		);



input CLK;

input RESET;

input PUSH;

input POP;

input FLUSH;

output [2:0] WRITE_DATA_COUNT;

output [2:0] READ_DATA_COUNT;

output [31:0] DATA_OUT;



input [31:0] DATA_IN;





reg [2:0] write_addr_count;

reg [2:0] read_addr_count;









parameter VCC = 1'b1;

parameter GND_1 = 1'b0;

parameter GND_16_BUS = 16'h0000;



RAMB4_S16_S16 high_word (

		.DOA (),

		.DOB (DATA_OUT[31:16]),

		.ADDRA ({5'b0, write_addr_count}),

		.CLKA (CLK),

		.DIA (DATA_IN[31:16]),

		.ENA (VCC),

		.RSTA (RESET),

		.WEA (PUSH),

		.ADDRB ({5'b0, read_addr_count}),

		.CLKB (CLK),

		.DIB (GND_16_BUS),

		.ENB (VCC),

		.RSTB (RESET),

		.WEB (GND_1)

		);



RAMB4_S16_S16 low_word (

		.DOA (),

		.DOB (DATA_OUT[15:0]),

		.ADDRA ({5'b0, write_addr_count}),

		.CLKA (CLK),

		.DIA (DATA_IN[15:0]),

		.ENA (VCC),

		.RSTA (RESET),

		.WEA (PUSH),

		.ADDRB ({5'b0, read_addr_count}),

		.CLKB (CLK),

		.DIB (GND_16_BUS),

		.ENB (VCC),

		.RSTB (RESET),

		.WEB (GND_1)

		);





// The address counter should increment on a pop and decrement on a push

always @(posedge CLK or posedge RESET)

	begin

	if (RESET)

		write_addr_count <= 8'h00;

	else

		begin

		if (FLUSH)

			write_addr_count <= 8'h00;

		else if (PUSH )

			write_addr_count [2:0] <= write_addr_count [2:0] + 1;

		end

	end



always @(posedge CLK or posedge RESET)

	begin

	if (RESET)

		read_addr_count <= 8'h00;

	else

		begin

		if (FLUSH)

			read_addr_count <= 8'h00;

		else if (POP)

			read_addr_count [2:0] <= read_addr_count [2:0] + 1;

		end

	end



assign WRITE_DATA_COUNT = write_addr_count [2:0];

assign READ_DATA_COUNT = read_addr_count [2:0];



endmodule

