/*--

-- File name 	: 	HIT_MACHINE.v

--             :

-- Title       : HIT_MACHINE.v

--             :

-- Purpose     :

--					:  This file contains a state machine which controls the READ and WRITE hits

--             :  to a register

--             :

--             :

-- Created On  : 08/23/00

--             :

-- Comments    : Design done by Insight Design Services

--             :

--					:

--					:

--             :

-- Conventions :

--					: Signal names ending in _int indicate the internal version of an output signal

--             : Tab size: 4

--   				:

--   				:

--   				:

-- Limitations : For use in Xilinx Virtex E FPGA

--   				:

-- Developers  : Duncan Allen

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version No:| Author    	| Changes Made: 						| Mod. Date:

--     v1.0     | da  			| First Written						| 08/23/00

--     v1.1     | da          | Modified for SDRAM controller  | 10/11/01

-- ------------------------------------------------------------------------*/



`timescale 1ns / 1ps



// This is a state machine for controlling the hits to the PCI registers

module REGISTER_HIT_MACHINE 

	(

		REGISTER_READ_HIT, 

		REGISTER_WRITE_HIT, 

		S_DATA, 

		CLK, 

		RESET,

		REGISTER_READ, 

		REGISTER_WRITE, 

		REGISTER_CLEAR,

		REGISTER_ABORT, 

		REGISTER_TERM, 

		REGISTER_READY

	);



input REGISTER_READ_HIT;         //Indicates a READ hit on BAR_0, the register BAR

input REGISTER_WRITE_HIT;          //Indicates a WRITE hit on BAR_0, the register BAR

input S_DATA;

input CLK;                  // The PCI_CLK

input RESET;                // Global RESET



output REGISTER_READ;        // A read is underway

output REGISTER_WRITE;       // A write is underway

output REGISTER_CLEAR;       // The read is finished and the register can be cleared if it is clear-on-read



output REGISTER_ABORT;

output REGISTER_TERM;

output REGISTER_READY;



reg REGISTER_READ;

reg REGISTER_WRITE;

reg REGISTER_CLEAR;



reg REGISTER_ABORT;

reg REGISTER_TERM;

reg REGISTER_READY;



parameter R_IDLE = 0, R_READ = 1, R_WRITE = 2;

reg [1:0] CURRENT_REGISTER_STATE, NEXT_REGISTER_STATE;



always @ (CURRENT_REGISTER_STATE or REGISTER_READ_HIT or REGISTER_WRITE_HIT  or S_DATA)

    begin //: REGISTER_HIT_MACHINE

        case (CURRENT_REGISTER_STATE)

            R_IDLE : begin

                // This is the idle state wating for a PCI hit

            REGISTER_READ <= 1'b0;

				REGISTER_WRITE <= 1'b0;

				REGISTER_ABORT <= 1'b0;

				REGISTER_TERM <= 1'b0;

				REGISTER_READY <= 1'b0;

                REGISTER_CLEAR <= 1'b0;



                if (REGISTER_READ_HIT == 1) NEXT_REGISTER_STATE <= R_READ;

                else if (REGISTER_WRITE_HIT == 1) NEXT_REGISTER_STATE <= R_WRITE;

                else NEXT_REGISTER_STATE <= R_IDLE;



            end



            R_READ : begin

                //There is a PCI Read

                // Both TERM and READY are set high because this will only be a single item read

                REGISTER_READ <= 1'b1;

                REGISTER_WRITE <= 1'b0;

                REGISTER_ABORT <= 1'b0;

                REGISTER_TERM <= 1'b1;

                REGISTER_READY <= 1'b1;



                if(S_DATA)

					begin

						NEXT_REGISTER_STATE <= R_READ;

                        REGISTER_CLEAR <= 1'b0;

                    end

                else

					begin

						NEXT_REGISTER_STATE <= R_IDLE;

                        REGISTER_CLEAR <= 1'b1;

                    end



            end



            R_WRITE : begin

                // There is a PCI Write

                // Both TERM and READY are set high because this will only be a single item write

                REGISTER_READ <= 1'b0;

                REGISTER_WRITE <= 1'b1;

                REGISTER_ABORT <= 1'b0;

                REGISTER_TERM <= 1'b1;

                REGISTER_READY <= 1'b1;

                REGISTER_CLEAR <= 1'b0;



                if (S_DATA) NEXT_REGISTER_STATE <= R_WRITE;

                else NEXT_REGISTER_STATE <= R_IDLE;



            end



            default : begin

                REGISTER_READ <= 1'b0;

                REGISTER_WRITE <= 1'b0;

                REGISTER_ABORT <= 1'b0;

                REGISTER_TERM <= 1'b0;

                REGISTER_READY <= 1'b0;

                REGISTER_CLEAR <= 1'b0;



                NEXT_REGISTER_STATE <= R_IDLE;

            end

        endcase

    end



always @ (posedge CLK or posedge RESET)

    begin

    if (RESET) CURRENT_REGISTER_STATE = R_IDLE;

    else CURRENT_REGISTER_STATE = NEXT_REGISTER_STATE;

    end







endmodule