//--------------------------------------------------------------------------------------------------

//             :

// File name   : PCI_DEVICE.V

//             :

// Title       : Generic PCI device Verilog model

//             :

// Comments    :  - design is separated into Target and Initiator Models

//             :  - Both models interface with RAM.  All accesses, except for configuration cycles,

//             :    directly read/write the RAM by default.

//             :

//             :  - In host mode, this device will generate the necessary initialization

//             :    configuration cycles

//             :

//             :

//             :  Wait State Control

//             :  ==================

//             :  This design uses a simple counter to generate wait states on a PCI access.

//             :  The counter counts the number of data phases and based on the following

//             :  parameters will insert wait states:

//             :

//             :  INSERT_WAITS   - boolean flag used to turn on/off the insertion of wait states

//             :  INITIAL_VALUE  - Initial value of counter

//             :  TERM_CNT       - terminal count, access will terminate when this value is reached

//             :  DURATION       - duration of wait state in number of clks, min 1, max 7

//             :  MOD_AMT        - modulus amount

//             :  RESULT         - test for modulus function

//             :

//             :  A wait state(s) is inserted when the following is true:

//             :

//             :     (COUNT % MODULUS) = RESULT

//             :

//             :  The duration of the wait is for "DURATION" number of PCI clock cycles.

//             :  Please note, there is no error checking of the wait state control parameters.

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Majid Kalsi

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//     v1.0     | mk        | First write                                              | 00/00/00

//--------------------------------------------------------------------------------------------------





`timescale 1 ns/1 ps





//-----------------------------------------------

//  Include Files

//-----------------------------------------------

`include "PCI_device_defines.v"





//-----------------------------------------------

//  Module declaration

//-----------------------------------------------

module PCI_DEVICE

(

   // command/control interface

   ICMD,                   // Initiator Command

   ICMD_ACK,               // Initiator Command Acknowledge

   ISTATE_out,                 // Initiator State

   IBUSY,                  // Initiator Busy

   TCMD,                   // Target Command

   TCMD_ACK,               // Target Command Acknowledge

   TSTATE_out,                 // Target State

   TBUSY,                  // Target Busy

   END_SIM,                // End of Simulation Flag



   // Initial Latency Control

   IRDY_INITIAL_LATENCY,   // IRDY Initial latency, min 0, max 7

   TRDY_INITIAL_LATENCY,   // TRDY Initial latency, min 0, max 15



   // Wait State Control, see description above

   INSERT_WAITS,           // Insert Wait States Flag

   INITIAL_VALUE,          // initial value of "count"

   TERM_CNT,               // termination count

   MOD_AMT,                // modulus amount

   RESULT,                 // test for mod function result

   DURATION,               // duration of waits, min 1, max 7



   // PCI signals

   CLK,

   RST_N,

   INTR_A,

   REQ_N,

   GNT_N,

   REQ64_N,

   ACK64_N,

   FRAME_N,

   IRDY_N,

   IDSEL,

   DEVSEL_N,

   TRDY_N,

   STOP_N,

   CBE,

   AD,

   PAR,

   PERR_N,

   SERR_N,

   PAR64

);



//-----------------------------------------------

//  Parameter declarations

//-----------------------------------------------



// General parameters

parameter pINST_NAME 			= "HOST";                  // instance name

parameter pLOG_FILE 				= "HOST.LOG";              // Log File

parameter pINTERRUPT_LOG_FILE = "INTERRUPT.LOG";         // Log File for the interrupt service routine

parameter pMEMORY_LOG_FILE   = "MEMORY.DMP";             // The memory file

parameter p66or33 				= `CLK66MHZ;               // default to 66 MHz timing model

parameter pBUS_WIDTH 			= `BIT32;                  // default to 64 bit data

parameter pADDRESSING 			= `BIT32;                  // default to 64 bit addressing

parameter pHOST_MODE 			= `TRUE;                   // default to HOST mode,

parameter pMEM_DEPTH 			= 1048576;                   // Memory Depth

parameter pMEM_BITS  			= 20;                      // Memory Depth in Bits



// -----------------------------------

// PCI configuration header parameters

// -----------------------------------



// Bar Registers

// =============

// These registers can be configured for both 32 bit and 64 bit

// addressing.  For 64 bit addressing, adjacent BARs are paired

// up.  ie,  BAR0 & BAR1

//           BAR2 & BAR3

//           BAR4 & BAR5

//

// BAR Parameter Definitions:

//

// pBARn_EN :     BAR enable

// pBARn_SIZE :   Address space size

// pBARn_FETCH :  PREFETCH supported

// pBARn_SPACE :  Address space, IO or MEMORY

// pBARn_ADDR :   Initial Base Address for given BAR

// pBARm_64ADDR : 64 bit addressing Enable

//

// NOTE:  n = 0,1,2,3,4,5

//        m = 0,2,4



parameter pBAR0_EN      = `TRUE;

parameter pBAR0_SIZE    = `SIZE4M;

parameter pBAR0_FETCH   = `NOFETCH;

parameter pBAR0_SPACE   = `MEM_SPACE;

parameter pBAR0_64ADDR  = `FALSE;

parameter pBAR0_ADDR    = 32'hA0000000;



parameter pBAR1_EN      = `FALSE;               // must be set to true for 64 bit addressing

parameter pBAR1_SIZE    = `SIZE4K;

parameter pBAR1_FETCH   = `NOFETCH;

parameter pBAR1_SPACE   = `IO_SPACE;

parameter pBAR1_ADDR    = 32'h0000000A;



parameter pBAR2_EN      = `TRUE;

parameter pBAR2_SIZE    = `GT_2G;

parameter pBAR2_FETCH   = `PREFETCH;

parameter pBAR2_SPACE   = `MEM_SPACE;

parameter pBAR2_64ADDR  = `TRUE;

parameter pBAR2_ADDR    = 32'h00000000;



parameter pBAR3_EN      = `TRUE;                // must be set to true for 64 bit addressing

parameter pBAR3_SIZE    = `SIZE64G;

parameter pBAR3_FETCH   = `PREFETCH;

parameter pBAR3_SPACE   = `MEM_SPACE;

parameter pBAR3_ADDR    = 32'h0000000B;



parameter pBAR4_EN      = `TRUE;

parameter pBAR4_SIZE    = `SIZE4K;

parameter pBAR4_FETCH   = `NOFETCH;

parameter pBAR4_SPACE   = `MEM_SPACE;

parameter pBAR4_64ADDR  = `TRUE;

parameter pBAR4_ADDR    = 32'h00000000;



parameter pBAR5_EN      = `TRUE;                // must be set to true for 64 bit addressing

parameter pBAR5_SIZE    = `LT_4G;

parameter pBAR5_FETCH   = `PREFETCH;

parameter pBAR5_SPACE   = `MEM_SPACE;

parameter pBAR5_ADDR    = 32'h0000000C;



// Command Register Bits, act as masks

parameter pFB2B_EN      = `FALSE;               // Fast back to back enable

parameter pSERR_EN      = `FALSE;               // SERR enable

parameter pSTEPPING     = `FALSE;               // Address/Data stepping enable

parameter pPERR_EN      = `FALSE;               // Parity Error Reporting enable

parameter pSNOOP_EN     = `FALSE;               // VGA Palette Snoop Enable

parameter pMWI_EN       = `FALSE;               // Memory Write & Invalide Enable

parameter pSPECIAL_EN   = `FALSE;               // Special Cycle enable

parameter pMASTER_EN    = `TRUE;                // Bus Mastering enable

parameter pMEM_EN       = `TRUE;                // Memory Access enable

parameter pIO_EN        = `TRUE;                // IO Access enable

// Status Register Bits, read only

parameter pCAPLIST      = `FALSE;               // Capabilities List Pointer capable

parameter pCLK66MHZ     = p66or33;              // 66 MHz capable

parameter pUDP         	= `FALSE;               // UDP supported

parameter pFB2B         = `FALSE;               // Fast back to back capable

parameter pDECODE       = `MEDIUM;              // DEVSEL decode timing

// Various IDs & CODEs

parameter pVENDOR_ID    = 16'hBEEF;

parameter pDEVICE_ID    = 16'hDEAD;

parameter pREV_ID       = 8'h10;

parameter pCLASS_CODE   = 24'hFF_00_00;

parameter pSUB_VID      = 16'hDEED;

parameter pSUB_ID       = 16'hABAD;

// Other header stuff

parameter pHEADER_TYPE  = 8'h00;                // Do not change, only type 00 supported

parameter pBIST_EN      = `FALSE;               // BIST enable

parameter pMIN_GNT      = 8'hFF;                // Minimum Grant Requirement

parameter pMAX_LAT      = 8'h00;                // Maximum Latency Requirement



// Misc Parameters, do not overwrite

parameter pTDEL = (p66or33) ? `TDEL_66 : `TDEL_33;

parameter pINT_PIN      = 8'h01;                // INTR_A only, do not change

parameter pCFG_CMD_MASK  = {{6{1'b0}}, pFB2B_EN, pSERR_EN, pSTEPPING, pPERR_EN, pSNOOP_EN, pMWI_EN,  pSPECIAL_EN, pMASTER_EN, pMEM_EN, pIO_EN};

parameter pCFG_STAT_REG = {pFB2B, pUDP, pCLK66MHZ, pCAPLIST, {3{1'b0}}};







//-----------------------------------------------

//  Port declarations

//-----------------------------------------------

input [3:0] ICMD;

output ICMD_ACK;

output [99:0]ISTATE_out;

output IBUSY;

input [3:0] TCMD;

output TCMD_ACK;

output [99:0]TSTATE_out;

output TBUSY;

input END_SIM;



// Initial Latency Control

input [3:0]IRDY_INITIAL_LATENCY;

input [3:0]TRDY_INITIAL_LATENCY;



// Wait State Control, see description above

input INSERT_WAITS;

input [3:0] INITIAL_VALUE;

input [3:0] TERM_CNT;

input [3:0] MOD_AMT;

input [3:0] RESULT;

input [3:0] DURATION;



// PCI signals

input CLK;

input RST_N;

input INTR_A;

output REQ_N;

input GNT_N;

input IDSEL;

inout FRAME_N;

inout IRDY_N;

inout DEVSEL_N;

inout TRDY_N;

inout STOP_N;

inout [(pBUS_WIDTH*4-1) : 0] CBE;

inout [(pBUS_WIDTH*32-1) : 0] AD;

inout PAR;

inout PERR_N;

inout SERR_N;

// PCI 64-Bit extension signals

inout REQ64_N;

inout ACK64_N;

inout PAR64;





//-----------------------------------------------

//  Variable declarations

//-----------------------------------------------

reg [(pBUS_WIDTH*32-1) : 0] ram0 [(pMEM_DEPTH-1) : 0];      // RAM for BAR 0

reg [(pBUS_WIDTH*32-1) : 0] ram1 [(pMEM_DEPTH-1) : 0];      // RAM for BAR 1

reg [(pBUS_WIDTH*32-1) : 0] ram2 [(pMEM_DEPTH-1) : 0];      // RAM for BAR 2

reg [(pBUS_WIDTH*32-1) : 0] ram3 [(pMEM_DEPTH-1) : 0];      // RAM for BAR 3

reg [(pBUS_WIDTH*32-1) : 0] ram4 [(pMEM_DEPTH-1) : 0];      // RAM for BAR 4

reg [(pBUS_WIDTH*32-1) : 0] ram5 [(pMEM_DEPTH-1) : 0];      // RAM for BAR 5

reg [31 : 0] bar_masks [5 : 0];                             // Bar Register Masks

reg [31 : 0] bars [5 : 0];                                  // Bar Registers

reg [5 : 0]  bar_hit;                                       // BAR hit

integer burst_end;                                          // burst length



// Configuration Header Registers/Signals

reg mdpe;                                                   // Master Data Parity Error, see CFG_STAT_REG

reg stabort;                                                // Signaled Target Abort, see CFG_STAT_REG

reg rtabort;                                                // Received Target Abort, see CFG_STAT_REG

reg rmabort;                                                // Received Master Abort, see CFG_STAT_REG

reg sserr;                                                  // Signaled System Error, see CFG_STAT_REG

reg dperr;                                                  // Detected Parity Error, see CFG_STAT_REG

reg [15:0] cfg_cmd_reg;                                     // Command Register

wire [15:0] cfg_cmd_mask;                                   // wire version of pCFG_CMD_MASK

wire[15:0] cfg_stat_reg;                                    // Status Register

reg [6:0] bist_reg;                                         // BIST Register

wire [7:0] bist;                                            // wire version of BIST register

reg [7:0] lat_timer_reg;                                    // Latency timer Register

reg [7:0] line_size_reg;                                    // Cache Line Size Register

reg [7:0] max_lat_reg;                                      // Maximum Latency Register

reg [7:0] min_gnt_reg;                                      // Minimum Grant Register

reg [7:0] int_line_reg;                                     // Interrupt Line Register



reg ICMD_ACK;

reg [(30*`SIZEOFCHAR - 1):0] ISTATE;

reg IBUSY;

reg TCMD_ACK;

reg [(30*`SIZEOFCHAR - 1):0] TSTATE;

reg TBUSY;



reg reg_req_n;

reg reg_frame_n;

reg reg_irdy_n;

reg reg_devsel_n;

reg reg_trdy_n;

reg reg_stop_n;

reg [(pBUS_WIDTH*4-1) : 0] reg_cbe;

reg [(pBUS_WIDTH*32-1) : 0] reg_ad;

reg reg_par;

reg reg_perr_n;

reg insert_perr;

reg reg_serr_n;

reg reg_req64_n;

reg reg_ack64_n;

reg reg_par64;



reg ad_oe;

reg cbe_oe;

reg par_oe;

reg par64_oe;

reg init_oe;

reg targ_oe;

reg perr_oe;

reg serr_oe;



reg interrupt_active; 	// This indicates whether the generic interrupt service routine should handle

                        // any interrupts or if the current test will handle interrupts



integer logfile;

integer interrupt_logfile;

integer memory_logfile;

real seed;

real random_num;





assign ISTATE_out = ISTATE [99:0];

assign TSTATE_out = TSTATE [99:0];



//-----------------------------------------------

// Define port hookup

//-----------------------------------------------

assign #pTDEL AD = ad_oe ? reg_ad : 64'bz;

assign #pTDEL CBE = cbe_oe ? reg_cbe : 'bz;

assign #pTDEL PAR = par_oe ? reg_par : 1'bz;

assign #pTDEL FRAME_N = init_oe ? reg_frame_n : 1'bz;

assign #pTDEL IRDY_N = init_oe ? reg_irdy_n : 1'bz;

assign #pTDEL DEVSEL_N = targ_oe ? reg_devsel_n : 1'bz;

assign #pTDEL TRDY_N = targ_oe ? reg_trdy_n : 1'bz;

assign #pTDEL STOP_N = targ_oe ? reg_stop_n : 1'bz;

assign #pTDEL PERR_N = perr_oe ? reg_perr_n : 1'bz;

assign #pTDEL SERR_N = serr_oe ? reg_serr_n : 1'bz;

assign #pTDEL REQ_N = reg_req_n;

assign #pTDEL REQ64_N = (init_oe && (pBUS_WIDTH == `BIT64)) ? reg_req64_n : 1'bz;

assign #pTDEL ACK64_N = (targ_oe && (pBUS_WIDTH == `BIT64)) ? reg_ack64_n : 1'bz;

assign #pTDEL PAR64 = (par64_oe && (pBUS_WIDTH == `BIT64)) ? reg_par64 : 1'bz;





//-----------------------------------------------

// PCI Parity Generation

//-----------------------------------------------

always @(posedge CLK) reg_par = (insert_perr) ? !(^ {AD[31:0], CBE[3:0]}) : (^ {AD[31:0], CBE[3:0]});

always @(posedge CLK) par_oe = ad_oe;

always @(posedge CLK) reg_par64 = (^ {AD[(pBUS_WIDTH*32-1): ((pBUS_WIDTH-1)*32)], CBE[(pBUS_WIDTH*4-1):((pBUS_WIDTH-1)*4)]});

always @(posedge CLK) par64_oe = ad_oe;





//-----------------------------------------------

// Define Internal signals

//-----------------------------------------------

assign cfg_cmd_mask = pCFG_CMD_MASK;

assign cfg_stat_reg = {dperr, sserr, rmabort, rtabort, stabort, pDECODE, mdpe, pCFG_STAT_REG};

assign bist = {pBIST_EN, bist_reg};





//-----------------------------------------------

//  INITIAL definition

//-----------------------------------------------

initial

begin : INIT_BLK

   integer i;

   reg [15:0] cfg_stat_reg;

   reg [(pBUS_WIDTH*32-1) : 0] temp;



   // Open logfile

   logfile = $fopen(pLOG_FILE);

   interrupt_logfile = $fopen(pINTERRUPT_LOG_FILE);

   memory_logfile = $fopen(pMEMORY_LOG_FILE);



   // Initialize Seed value for random number generator

   seed =  9.0;



   // initialize RAM to zeroes

   for(i = 0; i < pMEM_DEPTH; i = i + 1) begin

         ram0[i] = 64'b0;

         ram1[i] = 64'b0;

         ram2[i] = 64'b0;

         ram3[i] = 64'b0;

         ram4[i] = 64'b0;

         ram5[i] = 64'b0;

   end



   // initialize RAM0

 







   ICMD_ACK = 1'b0;

   TCMD_ACK = 1'b0;

   reg_req_n = 1'b1;

   reg_frame_n = 1'b1;

   reg_irdy_n = 1'b1;

   reg_devsel_n = 1'b1;

   reg_trdy_n = 1'b1;

   reg_stop_n = 1'b1;

   reg_cbe = 'b0;

   reg_ad = 'b0;

   reg_perr_n = 1'b1;

   reg_serr_n = 1'b1;

   reg_req64_n = 1'b1;

   reg_ack64_n = 1'b1;

   ad_oe = 1'b0;

   cbe_oe = 1'b0;

   init_oe = 1'b0;

   targ_oe = 1'b0;

   perr_oe = 1'b0;

   serr_oe = 1'b0;

   insert_perr = 1'b0;



   // load Configuration Header defaults

   mdpe     = 1'b0;                                 // Master Data Parity Error, see CFG_STAT_REG

   stabort  = 1'b0;                                 // Signaled Target Abort, see CFG_STAT_REG

   rtabort  = 1'b0;                                 // Received Target Abort, see CFG_STAT_REG

   rmabort  = 1'b0;                                 // Received Master Abort, see CFG_STAT_REG

   sserr    = 1'b0;                                 // Signaled System Error, see CFG_STAT_REG

   dperr    = 1'b0;                                 // Detected Parity Error, see CFG_STAT_REG

   cfg_cmd_reg = 16'b0;



   // Set burst_end flag

   burst_end = 1;



   bist_reg = 7'b0;                                  // Bist register[6:0] !

   lat_timer_reg = 8'h00;                           // Latency timer Register

	line_size_reg = 8'h00;                           // Cache Line Size Register;

	max_lat_reg = 8'h00;                             // Maximum Latency Register

	min_gnt_reg = 8'h00;                             // Minimum Grant Register

	int_line_reg = 8'h00;                            // Interrupt Line Register



	bar_hit = 6'b0;

	bars[0] = pBAR0_ADDR;

   bars[1] = pBAR1_ADDR;

   bars[2] = pBAR2_ADDR;

   bars[3] = pBAR3_ADDR;

   bars[4] = pBAR4_ADDR;

   bars[5] = pBAR5_ADDR;



   // load Bar Register Masks

   if(pBAR0_EN)

      bar_masks[0] = {pBAR0_SIZE, pBAR0_FETCH, pBAR0_64ADDR, 1'b0, pBAR0_SPACE};

   else

      bar_masks[0] = 32'h00000000;

   // end if



   if(pBAR1_EN)

      if(pBAR0_64ADDR)

         bar_masks[1] = pBAR1_SIZE;

      else

         bar_masks[1] = {pBAR1_SIZE, pBAR1_FETCH, 2'b00, pBAR1_SPACE};

      // end if

   else

      bar_masks[1] = 32'h00000000;

   // end if



   if(pBAR2_EN)

      bar_masks[2] = {pBAR2_SIZE, pBAR2_FETCH, pBAR2_64ADDR, 1'b0, pBAR2_SPACE};

   else

      bar_masks[2] = 32'h00000000;

   // end if



   if(pBAR3_EN)

      if(pBAR2_64ADDR)

         bar_masks[3] = pBAR3_SIZE;

      else

         bar_masks[3] = {pBAR3_SIZE, pBAR3_FETCH, 2'b00, pBAR3_SPACE};

      // end if

   else

      bar_masks[3] = 32'h00000000;

   // end if



   if(pBAR4_EN)

      bar_masks[4] = {pBAR4_SIZE, pBAR4_FETCH, pBAR4_64ADDR, 1'b0, pBAR4_SPACE};

   else

      bar_masks[4] = 32'h00000000;

   // end if



   if(pBAR5_EN)

      if(pBAR4_64ADDR)

         bar_masks[5] = pBAR5_SIZE;

      else

         bar_masks[5] = {pBAR5_SIZE, pBAR5_FETCH, 2'b00, pBAR5_SPACE};

      // end if

   else

      bar_masks[5] = 32'h00000000;

   // end if

end





// Initiator Loop

always @(posedge CLK or negedge RST_N or posedge END_SIM)

begin

   if(END_SIM == 1) begin

      $fclose(logfile);

      wait(!END_SIM);

   end

   if (RST_N == 0) begin

      IBUSY = `FALSE;

      ISTATE = "RESET";



      if(pHOST_MODE) begin

         wait (RST_N);

         @(posedge CLK);

         tINIT_S2PCI;     // Initialize 32 Bit Device

         //tDO_TEST1;

         //tDO_TEST2;

      end

      else begin

      	wait (RST_N);

      	ISTATE = "IDLE";

      end

   end

   else

      // if interrupt

      if(!INTR_A) begin

         @(posedge CLK);

         tSERVICE_INTERRUPT;

      end

      else

         if((ICMD != `NONE) && !TBUSY)begin

            if((ICMD == `TEST1) || (ICMD == `TEST2) || (ICMD == `TEST3) || (ICMD == `TEST4) || (ICMD == `TEST5) || (ICMD == `TEST6)  || (ICMD == `TEST7)

				|| (ICMD == `TEST8) || (ICMD == `TEST9)) begin

			

               ICMD_ACK = `TRUE;

               #1;   // wait 1 ns

               ICMD_ACK = `FALSE;

            end

		

            case(ICMD)

               `TEST1 :    tDO_TEST1;

               `TEST2 :    tDO_TEST2;

               `TEST3 :    tDO_TEST3;

               `TEST4 :    tDO_TEST4;

               `TEST5 :    tDO_TEST5;

               `TEST6 :    tDO_TEST6;

               `TEST7 :    tDO_TEST7;

               `TEST8 :    tDO_TEST8;

               `TEST9 :    tDO_TEST9;

               default :   ;// do nothing

            endcase

         end  

	

         // end if

      // end if

   // end if

end







// Target Loop

always @(posedge CLK or negedge RST_N)

begin

   if (RST_N == 0) begin

      TBUSY = `FALSE;

      TSTATE = "RESET";

      wait (RST_N);

      TSTATE = "IDLE";

   end

   else begin

      // if Address Phase

      if (!FRAME_N && IRDY_N ) begin

         case(CBE[3:0])

            `INT_ACK :   tSERVICE_INT_ACK_REQUEST;

            `SPECIAL :   tSERVICE_SPECIAL_REQUEST;

            `CFG_RD  :   if(IDSEL) tSERVICE_CFG_REQUEST;

            `CFG_WR  :   if(IDSEL) tSERVICE_CFG_REQUEST;

            `DAC     :   tSERVICE_DAC_REQUEST;

            default  :   // Single Address Cycle Request, ie 32 bit Address

					begin : SAC_ACCESS

               integer i;

               reg [31 : 0] bar_addr;

               reg [31 : 0] bar_mask;

               reg [5 : 0]  bar_ens;



               // determine enables,

               // -> not enabled when accessing a 64 bit BAR

               if((CBE[3:0] == `IO_RD) || (CBE[3:0] == `IO_WR)) begin

                  bar_ens[5] = pBAR5_EN && !pBAR4_64ADDR && (pBAR5_SPACE == `IO_SPACE);

                  bar_ens[4] = pBAR4_EN && !pBAR4_64ADDR && (pBAR4_SPACE == `IO_SPACE);

                  bar_ens[3] = pBAR3_EN && !pBAR2_64ADDR && (pBAR3_SPACE == `IO_SPACE);

                  bar_ens[2] = pBAR2_EN && !pBAR2_64ADDR && (pBAR2_SPACE == `IO_SPACE);

                  bar_ens[1] = pBAR1_EN && !pBAR0_64ADDR && (pBAR1_SPACE == `IO_SPACE);

                  bar_ens[0] = pBAR0_EN && !pBAR0_64ADDR && (pBAR0_SPACE == `IO_SPACE);

               end

               else if((CBE[3:0] == `MEM_RD) || (CBE[3:0] == `MEM_WR) ||

                       (CBE[3:0] == `MRM) || (CBE[3:0] == `MRL) || (CBE[3:0] == `MWI)) begin

                  bar_ens[5] = pBAR5_EN && !pBAR4_64ADDR && (pBAR5_SPACE == `MEM_SPACE);

                  bar_ens[4] = pBAR4_EN && !pBAR4_64ADDR && (pBAR4_SPACE == `MEM_SPACE);

                  bar_ens[3] = pBAR3_EN && !pBAR2_64ADDR && (pBAR3_SPACE == `MEM_SPACE);

                  bar_ens[2] = pBAR2_EN && !pBAR2_64ADDR && (pBAR2_SPACE == `MEM_SPACE);

                  bar_ens[1] = pBAR1_EN && !pBAR0_64ADDR && (pBAR1_SPACE == `MEM_SPACE);

                  bar_ens[0] = pBAR0_EN && !pBAR0_64ADDR && (pBAR0_SPACE == `MEM_SPACE);

               end

               else

                  bar_ens = 6'b0;





               for(i = 0; i < 6; i = i + 1) begin

                  bar_addr = bars[i];

                  bar_mask = bar_masks[i];

                  bar_hit[i] = (((AD[31:4] & bar_mask[31:4]) == bar_addr[31:4]) && bar_ens[i]);

               end



               if (bar_hit != 6'b000000)

                  tSERVICE_TARGET_REQUEST;

                  bar_hit = 6'b0;

               end

         endcase

      end

   end

end





//------------------------------------------------------------------------------

//  Instantiate Random Number Generator

//------------------------------------------------------------------------------

// RANDOM generates a random number with uniform distribution between 0 and 1.

// This function uses the multiplicative congruential method, using

// the following algorithm:

//       X(n+1) = A[X(n) + C] mod M

//       R(n) = X(n)/M

// where

//       M = 2**d, such that M(M-1) is close to 2**32 -1

//       C = 0

//       A = 8t +- 3, such that t is a positive integer and A

//                                            is close to 2**(d/2)

//       Seed(n) is a random number between 0 and M-1, with period M/4

//       Seed(0) is the seed for the sequence; must be the real representation

//            of an odd positive integer.

//       X(n) is a random number between 0 and 1

//------------------------------------------------------------------------------

always @(posedge CLK)

begin : RANDOM_NUM_GEN

   real num;



   tRANDOM(seed, num);

	random_num = 100.0 * num;



end





task tRANDOM;

 // task parameters

 inout  Seed;

 output num;



 // local variables

 integer M;

 integer A;

 integer iSeed;

 integer iTemp;

 real    num;

 real    Seed;



	begin

      M = 32768;

      A = 12869;



      iSeed = Seed;

	   iTemp = iSeed*A;

	   iSeed = iTemp/M;

	   iSeed = iTemp - iSeed*M;

	   Seed = iSeed;

	   num = Seed/M;

	end

endtask



//------------------------------------------------------------------------------

//------------------------------------------------------------------------------







//-----------------------------------------------

//  Add in Target & Initiator code

//-----------------------------------------------

`include "PCI_device_initiator.v"

`include "PCI_device_target.v"







endmodule