// Simple driver to drive the SpartanII-200 PCI Reference Design Board

#include "sii200.h"

#include "msglog.h"



// FPGA Register Locations (byte offset from BAR)

#define	LED_REG_OFFSET				0x0

#define SEVEN_SEG_REG_OFFSET		0x4

#define DIP_REG_OFFSET				0x18

#define PB_REG_OFFSET				0x24

#define TX_DATA_REG_OFFSET			0x30

#define RX_DATA_REG_OFFSET			0x34

#define UART_CTRL_REG_OFFSET		0x38

#define UART_STATUS_REG_OFFSET		0x3C



#define	FPGA						0		// BAR 0 for control functions



//==========================================================================//

// DriverEntry 

//   Purpose: The initial entry point for a WDM kernel device driver,

//            called at IRQL PASSIVE_LEVEL by the I/O Manager. 

//

//   Parameters:

//      IN DriverObject:  pointer to the driver object for the device

//      IN RegistryPath:  UniCode string with the driver's registry path name

//

//   Returns:

//      Status of the operation (STATUS_SUCCESS or failure code)

//==========================================================================//

NTSTATUS

DriverEntry (

  IN PDRIVER_OBJECT     DriverObject,

  IN PUNICODE_STRING    RegistryPath

  )

{

	NTSTATUS  status = STATUS_SUCCESS;

	PDEVICE_OBJECT        deviceObject = NULL;



	KdPrint (("sii200: DriverEntry\n"));



	// initialize the driver dispatch table

	DriverObject->DriverExtension->AddDevice			= AddDevice;

	DriverObject->DriverUnload							= Unload;

	DriverObject->MajorFunction[IRP_MJ_PNP]				= DispatchPnP;

	DriverObject->MajorFunction[IRP_MJ_POWER]			= DispatchPower;

	DriverObject->MajorFunction[IRP_MJ_CREATE]			= Dispatch;

	DriverObject->MajorFunction[IRP_MJ_CLOSE]			= Dispatch;

	DriverObject->MajorFunction[IRP_MJ_READ]			= Dispatch;

	DriverObject->MajorFunction[IRP_MJ_WRITE]			= Dispatch;

	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL]  = Control;

	DriverObject->DriverStartIo							= StartIo;

  

	// initialize driver variables



	// read or set driver values in the registry



	// save the RegistryPath



	KdPrint (("sii200: DriverEntry complete\n"));



	return status;

} // DriverEntry





//==========================================================================//

// Control 

//   Purpose: Allow BAR selection and setting of the byte offset

//            for reads and writes.  Called by the I/O Manager when

//            the IRP_MJ_DEVICE_CONTROL function code is encountered.

//

//   Parameters:

//      IN DeviceObject:  pointer to the device information structure

//      IN Irp:  pointer to the I/O Request Packet

//

//   Returns:

//      STATUS_SUCCESS or failure code

//==========================================================================//

NTSTATUS

Control (

  PDEVICE_OBJECT DeviceObject,

  PIRP Irp

  )

{

	ULONG				address;			// Register address

	PDEVICE_EXTENSION	deviceExtension;	// Device characteristics

	PIO_STACK_LOCATION	ioStack;			// Pointer to the current IRP stack

	ULONG				length;				// Length of the data buffer

	ULONG				loopLimit;			// Prevents endless loops

	PULONG				pBuffer;			// Pointer to the unsigned long parameter value

	ULONG				regValue;			// 32 bit register value

	NTSTATUS			status;				// return code



	KdPrint (("sii200: Control\n"));



	length = 0;



	status = STATUS_SUCCESS;

	deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;



	// Get the current stack and data buffer locations.

	ioStack = IoGetCurrentIrpStackLocation (Irp);

	pBuffer = (PULONG) Irp->AssociatedIrp.SystemBuffer;



	// Set the Base Address Register block.

    switch (ioStack->Parameters.DeviceIoControl.IoControlCode)  

	{

	case MDS_IOCTL_SET_BAR:

		if (*pBuffer == 0 || *pBuffer == 1)

		{

			RtlCopyMemory (&deviceExtension->bar, pBuffer, sizeof (ULONG));

			KdPrint (("Target memory set to BAR%ld.\n", deviceExtension->bar));

		}



		else

		{

			KdPrint(("Invalid BAR value: %ld.\n", *pBuffer));

			status = STATUS_INVALID_PARAMETER;

		}



		break;



	case MDS_IOCTL_SET_OFFSET: 

		if (*pBuffer <= deviceExtension->MemoryLength[deviceExtension->bar] - sizeof(ULONG))

		{

			RtlCopyMemory (&deviceExtension->MemoryOffset[deviceExtension->bar], 

			  pBuffer, sizeof (ULONG));



			// Align offset on ULONG boundary.

			deviceExtension->MemoryOffset[deviceExtension->bar] = 

			  deviceExtension->MemoryOffset[deviceExtension->bar] - 

			  deviceExtension->MemoryOffset[deviceExtension->bar] % sizeof(ULONG);

			KdPrint (("Memory offset for BAR%ld = %ld.\n", deviceExtension->bar,

			  deviceExtension->MemoryOffset[deviceExtension->bar]));

		}



		else

		{

			KdPrint(("Invalid offset value: %ld.\n", *pBuffer));

			status = STATUS_INVALID_PARAMETER;

		}



		break;



	case MDS_IOCTL_GET_USER_LED_STATE:

		// Get the register address and set the number of data elements to be read

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + LED_REG_OFFSET;

		length = 1;



		// Read the current LED state register.  

		*pBuffer = READ_REGISTER_ULONG ((PULONG) address);



		// The low order bit provides the LED state (0 = on;  1 = 0ff), so mask out

		// the rest of the register.

		*pBuffer &= 0x01;

		break;



	case MDS_IOCTL_SET_USER_LED_STATE:

		// Get the register address and set the number of data elements to be written

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + LED_REG_OFFSET;

		length = 1;



		// pBuffer contains the value to write to the register.  Note that we

		// write a full 32 bits here, but that's OK in this case since bits

		// 1..31 are unused in this register.

		WRITE_REGISTER_ULONG ((PULONG) address, *pBuffer);

		break;



	case MDS_IOCTL_GET_7_SEGMENT_STATE:

		// Get the register address and set the number of data elements to be read

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + SEVEN_SEG_REG_OFFSET;

		length = 1;

			



		if (*pBuffer != 0)

		{

			*pBuffer = READ_REGISTER_ULONG ((PULONG) address);		// Read both registers

			*pBuffer &= 0xF;										// Keep only the lower register

		}



		else

		{

			*pBuffer = READ_REGISTER_ULONG ((PULONG) address);		// Read both registers

			*pBuffer &= 0xF0;										// Keep only the upper register

			*pBuffer >>= 4;											// Return reg value in lower 4 bits

		}



		break;



 	case MDS_IOCTL_SET_7_SEGMENT_STATE:

		// Get the register address and set the number of data elements to be written

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + SEVEN_SEG_REG_OFFSET;

		length = 1;



		// pBuffer contains two long values.  The first is the index (0 or 1) of the 

		// 7-segment display, and the second is the value to set the display to.

		if (*pBuffer != 0)

		{

			*pBuffer = READ_REGISTER_ULONG ((PULONG) address);	// Get the current register settings

			*pBuffer &= 0xF0;									// Clear all but register 2

			pBuffer += 1;										// Advance to new value for register 1

			*pBuffer &= 0xF;									// Clear all but the new value

			*pBuffer |= *(pBuffer - 1);							// Combine new reg 1 & old reg 2 values

		}



		else

		{

			*pBuffer = READ_REGISTER_ULONG ((PULONG) address);	// Get the current register settings

			*pBuffer &= 0xF;									// Clear all but register 1

			pBuffer += 1;										// Advance to new value for register 2

			*pBuffer <<= 4;										// Shift new value to reg 2 position

			*pBuffer &= 0xF0;									// Clear all but the new value

			*pBuffer |= *(pBuffer - 1);							// Combine new reg 2 & old reg 1 values

		}

			

		WRITE_REGISTER_ULONG ((PULONG) address, *pBuffer);		// Write both registers

		break;



 	case MDS_IOCTL_GET_DIP_STATE:

		// Get the register address and set the number of data elements to be read

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + DIP_REG_OFFSET;

		length = 1;



		// Read the current LED state register.  

		*pBuffer = READ_REGISTER_ULONG ((PULONG) address);



		// Mask out the top three bytes (unused) in the word.

		*pBuffer &= 0xFF;

		break;



	case MDS_IOCTL_GET_PB_STATE:

		// Get the register address and set the number of data elements to be read

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + PB_REG_OFFSET;

		length = 1;



		// Read the current Push Button state register.  

		*pBuffer = READ_REGISTER_ULONG ((PULONG) address);



		// Mask out all the bits except for bit 0.

		*pBuffer &= 0x01;

		break;



	case MDS_IOCTL_SET_UART_CTRL:

		// Get the register address and set the number of data elements to be written

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + UART_CTRL_REG_OFFSET;

		length = 1;



		// pBuffer contains the values to write to the register.  The first value is the

		// RX interrupt level (this means that an interrupt occurs after receiving n chars),

		// and the second value is desired baud rate.

		// Bits 23-16 contain the RX interrupt, and bits 15-0 have the baud rate in the

		// control register.



		if (*pBuffer > 255 || *pBuffer < 0)

			*pBuffer = 0;



		*pBuffer <<= 16;			// Shift the RX interrupt level into the correct reg bits.



		if (*(pBuffer + 1) > 115200 || *(pBuffer + 1) < 9600)

			*(pBuffer + 1) = 9600;



		// The speed select value in the register is obtained by dividing 16X the desired baud rate

		// into 33333333.

		*(pBuffer + 1) = 33333333 / (16 * *(pBuffer + 1));

		*(pBuffer + 1) &= 0xFFFF;	



		// Combine the two values into an image of the register pattern and write the register.

		*pBuffer |= *(pBuffer + 1);

		WRITE_REGISTER_ULONG ((PULONG) address, *pBuffer);

		break;



	case MDS_IOCTL_ECHO_RX:

		// Get the register address and set the number of data elements to be written

		// for use in the IoStatus.Information field on return

		address = (ULONG) deviceExtension->MemoryStart[FPGA] + UART_STATUS_REG_OFFSET;

		length = 1;

		*pBuffer = 0;



		// Read the status register.

		regValue = READ_REGISTER_ULONG ((PULONG) address);



		if ((regValue & RX_FIFO_OVERFLOW) == RX_FIFO_OVERFLOW)  

		{

			*pBuffer = ERR_RX_FIFO_OVERFLOW;

		}



		else if ((regValue & RX_READY) != RX_READY)

		{

			*pBuffer = ERR_RX_NO_DATA_RECEIVED;

			regValue = 0;

		}



		// As long as we have data ready, copy it from the Receive FIFO and write it to the

		// Transmit register.  The data is automatically propagated by the hardware to the FIFO

		// if it arrives faster than it can be transmitted.

		while ((regValue & RX_READY) == RX_READY)	// Data ready

		{

			// Read a byte from the Receive FIFO.

			address = (ULONG) deviceExtension->MemoryStart[FPGA] + RX_DATA_REG_OFFSET;

			regValue = READ_REGISTER_ULONG ((PULONG) address) & 0xFF;

			

			if (*pBuffer != ERR_ECHO_TIMEOUT)

			{

				// Set a limit value on looping while waiting for a change in status to prevent the

				// device driver from entering an endless loop if there is a HW problem.

				loopLimit = 1000000;



				// Update the address to look at the UART Status register.

				address = (ULONG) deviceExtension->MemoryStart[FPGA] + UART_STATUS_REG_OFFSET;

				

				// Check the UART Status.  While the transmit bit is zero, we cannot write to the

				// transmit buffer, so loop waiting for this condition to change.  If it doesn't ever

				// change, we set an error code and stop processing.

				while ((READ_REGISTER_ULONG ((PULONG) address) & TX_READY) != TX_READY && loopLimit > 0)

					loopLimit -= 1;



				if (loopLimit <= 0)

				{

					*pBuffer = ERR_ECHO_TIMEOUT;			

				}



				else

				{

					// Write the data to the transmit register.  Increment the

					// length count to keep track of the number of bytes we echoed.

					address = (ULONG) deviceExtension->MemoryStart[FPGA] + TX_DATA_REG_OFFSET;

					WRITE_REGISTER_ULONG ((PULONG) address, regValue);

					*pBuffer += 1;

				}

			}



			// Check the status register to see if there is more data to read.

			address = (ULONG) deviceExtension->MemoryStart[FPGA] + UART_STATUS_REG_OFFSET;

			regValue = READ_REGISTER_ULONG ((PULONG) address);

		}



		break;



    default:

		// Set failure status for unrecognized control codes.

		KdPrint(("Received invalid I/O Control code.\n"));

		status = STATUS_NOT_IMPLEMENTED;

	}



	Irp->IoStatus.Status = status;

	Irp->IoStatus.Information = length * sizeof(ULONG);		// This must be set or no info will be returned.



	//	Tell the I/0 Manager that everything has been done.

    IoCompleteRequest( Irp, IO_NO_INCREMENT );



	return status;

}



// IRQL PASSIVE_LEVEL

NTSTATUS

Dispatch (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	LARGE_INTEGER		byteOffset;			// bytes from base addr. to begin op.

	PDEVICE_EXTENSION   deviceExtension;	// device characteristics

	ULONG				length;				// number of bytes to read/write

	PIO_STACK_LOCATION	ioStack;			// Pointer to the current IRP stack

	NTSTATUS			status;				// completion code



	// Start by assuming success and deferred processing.

	status = STATUS_PENDING;



	// Retrieve our IRP and device characteristics.

	ioStack = IoGetCurrentIrpStackLocation(Irp);

	deviceExtension = DeviceObject->DeviceExtension;

	

	// Select the operation and set the transfer parameters.

	switch(ioStack->MajorFunction)

	{

	case IRP_MJ_CREATE:

		LogMsg(DeviceObject, L"File object successfully created");

		Irp->IoStatus.Status = STATUS_SUCCESS;

		Irp->IoStatus.Information = 0;

		IoCompleteRequest(Irp, IO_NO_INCREMENT);

		return STATUS_SUCCESS;



	case IRP_MJ_CLOSE:

		LogMsg(DeviceObject, L"File object successfully closed");

		Irp->IoStatus.Status = STATUS_SUCCESS;

		Irp->IoStatus.Information = 0;

		IoCompleteRequest(Irp, IO_NO_INCREMENT);

		return STATUS_SUCCESS;



	case	IRP_MJ_READ:

		byteOffset = ioStack->Parameters.Read.ByteOffset;

		length = ioStack->Parameters.Read.Length;

		break;



	case	IRP_MJ_WRITE:

		byteOffset = ioStack->Parameters.Write.ByteOffset;

		length = ioStack->Parameters.Write.Length;

		break;



	default:	// Invalid 

		status = STATUS_NOT_IMPLEMENTED;

		Irp->IoStatus.Status = status;

		break;

	}

 

	// Align transfer length on ULONG boundary.

	if (length % sizeof(ULONG) != 0)

	{

		length = length + sizeof(ULONG) - length % sizeof(ULONG);

	}



	KdPrint(("length = %ld bytes, offset = %ld bytes\n", 

	  length, deviceExtension->MemoryOffset[deviceExtension->bar]));

	KdPrint(("Maximum length = %ld bytes\n", 

	  deviceExtension->MemoryLength[deviceExtension->bar]));



	// Validate the incoming parameters

	if (byteOffset.QuadPart != 0)		

	{

		// We use IOCTRL to set the offset, so disallow an offset set here.

		status = STATUS_INVALID_PARAMETER;

	}



	// Check for errors.

	// If we're pending and there are no errors, we queue the IRP for

	// deferred processing by the I/O Manager.

	if ( status != STATUS_PENDING || 

		 deviceExtension->MemoryOffset[deviceExtension->bar] + length > 

		 deviceExtension->MemoryLength[deviceExtension->bar])

	{



		if (status == STATUS_PENDING)

		{

			// Set the status for failure.

			status = STATUS_INVALID_PARAMETER;

		}



		// Complete the I/O Request.

		Irp->IoStatus.Status = status;

		Irp->IoStatus.Information = 0;

		IoCompleteRequest(Irp, IO_NO_INCREMENT);

	}



	else

	{

		// Mark the Irp for deferred processing.

		IoMarkIrpPending(Irp);



		// Queue the Irp for processing in the StartIo function, which

		// will be called by the I/O Manager when the DPC is dequeued.

		IoStartPacket(DeviceObject, Irp, NULL, NULL);

	}



    return status;

} // Dispatch



//===========================================================================

// LogMsg

//   Purpose:  Sends a message to the System Event Log.  Note that callers

//             of this function must be running at IRQL <= DISPATCH_LEVEL,

//             due to the constraints of "IoWriteErrorLogEntry".

//

//   Parameters:

//       IN deviceObject: pointer to the device information structure

//       IN logMsg: pointer to a wide char array containing the message.

//

//   Returns:

//       None.

//===========================================================================

VOID

LogMsg(

	IN PDEVICE_OBJECT deviceObject,		//  Device Object

	IN const wchar_t *logMsg			//  Message for event log

	)

{

	PIO_ERROR_LOG_PACKET	errorLogEntry;

	ULONG					len;



	// Verify that we can handle the current IRQL level.  If not, 

	// do nothing and return, as recommended in the IoWriteErrorLogEntry

	// documentation.

	if (KeGetCurrentIrql() > DISPATCH_LEVEL)

		return;



	// Verify that we have a device object.  If not, just return. 

	if (deviceObject == NULL)

		return;



	// Calculate the message length.

	len = (UCHAR)wcslen(logMsg)*sizeof(WCHAR) + sizeof(WCHAR);



	// Allocate the log packet and fill in the required information.

	errorLogEntry  = (PIO_ERROR_LOG_PACKET)

		IoAllocateErrorLogEntry(deviceObject, 

			(UCHAR)(sizeof(IO_ERROR_LOG_PACKET) + len));

	errorLogEntry->ErrorCode = (ULONG) MSGLOG_INFORMATION;

	errorLogEntry->NumberOfStrings = 1;

	errorLogEntry->StringOffset = 

		FIELD_OFFSET(IO_ERROR_LOG_PACKET, DumpData);



	// Copy the insertion string to the packet.

	RtlCopyMemory(&errorLogEntry->DumpData[0], logMsg, len);



	// Write the packet to the event log.

	IoWriteErrorLogEntry(errorLogEntry);

}



//==========================================================================//

// StartIo

//   Purpose: Read/Write data from/to the onboard SDRAM.

//

//   Parameters:

//      IN DeviceObject:  pointer to the device information structure

//      IN Irp:  pointer to the I/O Request Packet

//

//   Returns:

//      If function succeeds, STATUS_SUCCESS;  otherwise a failure code.

//==========================================================================//

NTSTATUS

StartIo(

    IN PDEVICE_OBJECT DeviceObject,

    IN PIRP Irp

    )

{    

	

	ULONG				address;				// Location of 1st memory byte

	PDEVICE_EXTENSION   deviceExtension;		// device characteristics

	PIO_STACK_LOCATION	ioStack;				// Pointer to the current I/O request

	ULONG				length;					// Length of the data buffer

	ULONG		*		pBuffer;				// Pointer to the system data buffer

	NTSTATUS			status;					// return code



	deviceExtension = DeviceObject->DeviceExtension;



	// Locate the current stack location and data buffer.

	ioStack = IoGetCurrentIrpStackLocation (Irp);

    pBuffer = (PULONG) Irp->AssociatedIrp.SystemBuffer;

	//pBuffer = (PULONG) MmGetSystemAddressForMdl(Irp->MdlAddress);



	// Calculate the number of DATA ELEMENTS (not bytes) to be transferred.

    length = ioStack->Parameters.Write.Length / sizeof (ULONG);



	// Get the register address.

    address = (ULONG) deviceExtension->MemoryStart[deviceExtension->bar] + 

		deviceExtension->MemoryOffset[deviceExtension->bar];



	// Initialize the I/O result status and transfer length.  If anything goes

	// wrong later, both values will be redefined.

	status = STATUS_SUCCESS;

	Irp->IoStatus.Status = STATUS_SUCCESS;

	Irp->IoStatus.Information = length * sizeof(ULONG);



	switch(ioStack->MajorFunction)

	{

	case	IRP_MJ_READ:

		READ_REGISTER_BUFFER_ULONG ((PULONG) address, pBuffer, length);

		break;



	case	IRP_MJ_WRITE:

		WRITE_REGISTER_BUFFER_ULONG ((PULONG) address, pBuffer, length);

		break;



	default:	// Invalid 

		KdPrint (("ERROR: Invalid I/O operation detected in StartIo."));

		Irp->IoStatus.Status = STATUS_NOT_IMPLEMENTED;

		Irp->IoStatus.Information = 0;

		status = STATUS_NOT_IMPLEMENTED;

		break;

	}



	//	Tell the I/0 that everything has been done.

    IoCompleteRequest( Irp, IO_NO_INCREMENT );



	// Have the I/O Manager propagate execution by looking for the next IRP.

	// None will be found, at which point the I/O Manager will clear the BUSY

	// flag for the device and execution will continue.  Failure to make this

	// call will cause the calling application to hang.

	IoStartNextPacket(DeviceObject, FALSE);



    return status;

}



// IRQL PASSIVE_LEVEL

VOID

Unload (

  IN PDRIVER_OBJECT     DriverObject

  )

{

	PAGED_CODE ();



	KdPrint (("sii200: DriverUnload\n"));



	return;

} // Unload







